@@ -25,7 +25,7 @@ setAs("dsyMatrix", "dsTMatrix",
           uplo <- from@uplo
           ij <- ij[if(uplo == "U") ij[,1] <= ij[,2] else ij[,1] >= ij[,2] ,
                    , drop = FALSE]
-          new("dsTMatrix", i = ij[,1] - 1:1, j = ij[,2] - 1:1,
+          new("dsTMatrix", i = ij[,1] - 1L, j = ij[,2] - 1L,
               x = as.vector(m[ij]), uplo = uplo,
               Dim = from@Dim, Dimnames = from@Dimnames)
       })
